@@ -1,6 +1,6 @@
 #--------------------------------------------------------------------------
 #     This file is part of OASA - a free chemical python library
-#     Copyright (C) 2003 Beda Kosata <beda@zirael.org>
+#     Copyright (C) 2003-2008 Beda Kosata <beda@zirael.org>
 
 #     This program is free software; you can redistribute it and/or modify
 #     it under the terms of the GNU General Public License as published by
@@ -405,6 +405,8 @@ class graph( object):
     """returns all cycles found in the graph as sets of edges;
     this version of the algorithm strips all non-cyclic (bridge) edges
     and then searches for cycles in the rest"""
+    if not self.contains_cycle():
+      return Set()
     self.temporarily_strip_bridge_edges()
 
     to_go = Set( [v for v in self.vertices if v.degree])
@@ -466,6 +468,7 @@ class graph( object):
     other cycles in graph are guaranteed to be combinations of them.
     Gasteiger J. (Editor), Engel T. (Editor), Chemoinformatics : A Textbook, John Wiley & Sons 2001,
     ISBN 3527306811, 174."""
+    assert self.is_connected()
     ncycles = len( self.edges) - len( self.vertices) + 2 - len( list( self.get_connected_components()))
 
     # check if the graph is connected, don't know if we should do it...
@@ -473,10 +476,14 @@ class graph( object):
       warnings.warn( "The number of edges is smaller than number of vertices-1, the molecule must be disconnected, which means there is something wrong with it.", UserWarning, 3)
       ncycles = 0
 
+    # trivial case of linear molecule
+    if ncycles == 0:
+      return Set()
+
     # the code itself
     self.temporarily_strip_bridge_edges()
-
     cycles = Set()
+
     vs = [v for v in self.vertices if v.degree]
     while vs and len( cycles) < ncycles:
       new_cycles = Set()
@@ -640,6 +647,39 @@ class graph( object):
       except KeyError:
         pass
 
+  def mark_edges_with_distance_from( self, e1):
+    for e in self.edges:
+      try:
+        del e.properties_['d']
+      except KeyError:
+        pass
+    marked = Set( [e1])
+    new = Set( [e1])
+    dist = 0
+    e1.properties_['dist'] = dist
+    while new:
+      new_new = Set()
+      dist += 1
+      for e in new:
+        for ne in e.get_neighbor_edges():
+          if not ne in marked:
+            ne.properties_['dist'] = dist
+            new_new.add( ne)
+      new = new_new
+      marked |= new
+
+  def get_path_between_edges( self, e1, e2):
+    self.mark_edges_with_distance_from( e1)
+    if not "dist" in e2.properties_:
+      return None
+    else:
+      path = [e2]
+      _e = e2
+      for i in range( e2.properties_['dist']-1, -1, -1):
+        _e = [ee for ee in _e.get_neighbor_edges() if ee.properties_['dist'] == i][0]
+        path.append( _e)
+      return path
+  
 
   def _gen_diameter_progress( self):
     """this generator iteratively generates graph diameter during its computation,
@@ -661,7 +701,6 @@ class graph( object):
     #best_path.reverse()
 
 
-
   def _get_width_from_vertex( self, v):
     """returns width of the graph as calculated from vertex v"""
     d = 0
@@ -670,17 +709,12 @@ class graph( object):
     while to_mark:
       marked_before = marked
       marked = to_mark
-
       to_mark_next = Set( reduce( operator.add, [i.get_neighbors() for i in to_mark], []))
-
       to_mark = to_mark_next - marked - marked_before
       d += 1
-
     return d-1
 
 
-
-
   def get_diameter( self):
     d = self._get_cache( "diameter")
     if not d:
@@ -750,15 +784,17 @@ class graph( object):
     """returns a induced subgraph that is newly created and can be therefore freely
     changed without worry about the original."""
     sub = self.create_graph()
-    r1, r2 = [], []
+    vertex_map = {}
+    i = 0
     for v in vertices:
-      r1.append( v)
-      sub.add_vertex()
+      new_v = v.copy()
+      sub.add_vertex( new_v)
+      vertex_map[v] = i
+      i += 1
     for e in edges:
+      new_e = e.copy()
       v1, v2 = e.get_vertices()
-      i1 = r1.index( v1)
-      i2 = r1.index( v2)
-      sub.add_edge( i1, i2)
+      sub.add_edge( vertex_map[v1], vertex_map[v2], new_e)
     return sub
 
   def defines_connected_subgraph_e( self, edges):
@@ -861,7 +897,12 @@ class graph( object):
       i1, i2 = map( int, line.split())
       self.add_edge( self.vertices[i1], self.vertices[i2])
 
-
+  def path_exists( self, a1, a2):
+    self.mark_vertices_with_distance_from( a1)
+    if 'd' in a2.properties_:
+      return True
+    else:
+      return False
 
   ## STATIC METHODS
 
