@@ -1,6 +1,6 @@
 #--------------------------------------------------------------------------
 #     This file is part of OASA - a free chemical python library
-#     Copyright (C) 2003-2007 Beda Kosata <beda@zirael.org>
+#     Copyright (C) 2003-2008 Beda Kosata <beda@zirael.org>
 
 #     This program is free software; you can redistribute it and/or modify
 #     it under the terms of the GNU General Public License as published by
@@ -36,7 +36,7 @@ import coords_generator
 from oasa_exceptions import oasa_not_implemented_error, oasa_inchi_error, oasa_unsupported_inchi_version_error
 import select
 import sys
-
+from stereochemistry import cis_trans_stereochemistry
 
 class inchi( plugin):
 
@@ -48,7 +48,13 @@ class inchi( plugin):
   proton_acceptors = ['P','N','S','O']
   electron_donors = ['O','N','S','P']
   electron_acceptors = ['B']
-  
+
+  stereo_remap = {"+": cis_trans_stereochemistry.OPPOSITE_SIDE,
+                  "-": cis_trans_stereochemistry.SAME_SIDE,
+                  "?": cis_trans_stereochemistry.UNDEFINED,
+                  "u": cis_trans_stereochemistry.UNDEFINED,
+                  }
+
 
   def __init__( self, structure=None):
     self.structure = structure
@@ -103,12 +109,16 @@ class inchi( plugin):
 
 
 
-
   def get_layer( self, prefix):
     for l in self.layers:
       if l.startswith( prefix):
         return l[1:]
 
+  def get_atom_with_inchi_number( self, number):
+    assert type( number) == int
+    for v in self.structure.vertices:
+      if v.properties_['inchi_number'] == number:
+        return v
 
   def read_inchi( self, text):
     try:
@@ -161,6 +171,7 @@ class inchi( plugin):
       #self._deal_with_valencies()
       self.compensate_for_forced_charges()
       self.structure.add_missing_bond_orders()
+      #self.read_double_bond_stereo_layer()
 
       # here we check out if the molecule seems ok
       fvs = [v for v in self.structure.vertices if v.free_valency]
@@ -452,6 +463,38 @@ class inchi( plugin):
       assert p < old_p
 
 
+  def read_double_bond_stereo_layer( self):
+    def get_lowest_numbered_neighbor( atom, verbotten_atom):
+      neighs = [(n.properties_.get('inchi_number',1000000), n)
+                for n in atom.neighbors
+                if n is not verbotten_atom and "inchi_number" in n.properties_]
+      if neighs:
+        neighs.sort( reverse=True)
+        return neighs[0][1]
+      else:
+        # we take the only one that is there (if any)
+        neighs = [n for n in atom.neighbors if n is not verbotten_atom]
+        if neighs:
+          return neighs[0]
+        else:
+          raise oasa_inchi_error( "No neigbors on atom with stereo information!")
+        
+
+    
+    layer = self.get_layer( "b")
+    if not layer:
+      return
+    for a1,a2,sign in re.findall( "(\d+)-(\d+)([-+?u])", layer):
+      atom1 = self.get_atom_with_inchi_number( int( a1))
+      atom2 = self.get_atom_with_inchi_number( int( a2))
+      bond = self.structure.get_edge_between( atom1, atom2)
+      # we need neighbors with lowest inchi_number
+      neigh1 = get_lowest_numbered_neighbor( atom1, atom2)
+      neigh2 = get_lowest_numbered_neighbor( atom2, atom1)
+      st = cis_trans_stereochemistry( references=[neigh1,atom1,atom2,neigh2],
+                                      value=self.stereo_remap[sign])
+      self.structure.add_stereochemistry( st)
+
 
 
   def _valency_to_charge( self, v, charge):
@@ -720,9 +763,42 @@ class inchi( plugin):
 #        elif x == 0:
 #          break
         
-    
 
-def generate_inchi_and_inchikey( m, program=None, fixed_hs=False):
+def _run_command( command, input):
+  if os.name == "nt":
+    return _run_command_nt( command, input)
+  p = popen2.Popen4( command)
+  out, inp = p.fromchild, p.tochild
+  inp.write( input)
+  inp.close()
+  outf = out.fileno()
+  result = ""
+  try:
+    while 1:
+      ready = select.select( [outf], [], [])
+      if ready[0]:
+        text = os.read( ready[0][0], 1024)
+        if not text:
+          break
+        result += text
+      time.sleep(0.01)
+  finally:
+    out.close()
+    p.poll()
+    del p
+  return result
+
+def _run_command_nt( command, input):
+  out, inp = popen2.popen4( command)
+  inp.write( input)
+  inp.close()
+  text = out.read()
+  return text
+
+
+def generate_inchi_and_inchikey( m, program=None, fixed_hs=True, ignore_key_error=False):
+  """ignore the case when InChIKey cannot be generated to some reason
+  (no mhash library and old InChI program)"""
   if not program:
     import config
     program = config.Config.inchi_binary_path
@@ -732,44 +808,43 @@ def generate_inchi_and_inchikey( m, program=None, fixed_hs=False):
   else:
     options = "-AUXNONE -STDIO -Key" + (fixed_hs and " -FixedH" or "")
   command = ' '.join( ('"'+os.path.abspath( program)+'"', options))
-  if os.name == "nt":
-    out, inp = popen2.popen4( command)
-  else:
-    p = popen2.Popen4( command)
-    out, inp = p.fromchild, p.tochild
-  inp.write( mf)
-  inp.close()
+  text = _run_command( command, mf)
   inchi = ""
   key = ""
   warnings = []
-  for line in out.readlines():
+  for line in text.splitlines():
     if line.startswith( "Warning"):
       warnings.append(line.strip() + "\n")
     elif line.startswith( "End of file detected"):
       pass
     elif line.startswith( "InChIKey="):
-      key = line.strip()
+      key = line.strip()[9:]
       break
     elif line.startswith( "InChI="):
       inchi = inchi + line.strip()
     elif line.startswith( "Error"):
       break
-  out.close()
-
-  if os.name != "nt":
-    p.poll()
-    del p
-
   if not inchi:
     raise oasa_inchi_error( "InChI program did not create any output InChI")
+  if not key:
+    # probably old version of the InChI software
+    try:
+      import inchi_key
+    except ImportError:
+      if ignore_key_error:
+        key = None
+      else:
+        raise oasa_inchi_error( "InChIKey could not be generated - inchi_key module failed to load properly.")
+    else:
+      key = inchi_key.key_from_inchi( inchi)
   return inchi, key, warnings
 
 
-def generate_inchi( m, program=None, fixed_hs=False):
-  inchi, key, warnings = generate_inchi_and_inchikey( m, program=program, fixed_hs=fixed_hs)
+def generate_inchi( m, program=None, fixed_hs=True):
+  inchi, key, warnings = generate_inchi_and_inchikey( m, program=program, fixed_hs=fixed_hs, ignore_key_error=True)
   return inchi, warnings
 
-def generate_inchi_key( m, program=None, fixed_hs=False):
+def generate_inchi_key( m, program=None, fixed_hs=True):
   inchi, key, warnings = generate_inchi_and_inchikey( m, program=program, fixed_hs=fixed_hs)
   return key, warnings
 
@@ -790,7 +865,7 @@ def text_to_mol( text, include_hydrogens=True, mark_aromatic_bonds=False, calc_c
     mol = inc.structure
     #mol.add_missing_bond_orders()
     if not include_hydrogens:
-      mol.remove_all_hydrogens()
+      mol.remove_unimportant_hydrogens()
     if calc_coords:
       coords_generator.calculate_coords( mol, bond_length=calc_coords)
     if mark_aromatic_bonds:
@@ -807,7 +882,7 @@ def file_to_mol( f):
   return text_to_mol( f.read())
 
 
-def mol_to_text( mol, program=None, fixed_hs=False):
+def mol_to_text( mol, program=None, fixed_hs=True):
   return generate_inchi( mol, program=program, fixed_hs=fixed_hs)[0]
 
 
@@ -832,7 +907,7 @@ if __name__ == '__main__':
   def main( text, cycles):
     t1 = time.time()
     for jj in range( cycles):
-      mol = text_to_mol( text, calc_coords=False, include_hydrogens=False)
+      mol = text_to_mol( text, calc_coords=True, include_hydrogens=False)
       print map( str, [b for b in mol.bonds if b.order == 0])
       print "  smiles: ", smiles.mol_to_text( mol)
       print "  inchi:  ", mol_to_text( mol, fixed_hs=True)
@@ -842,7 +917,7 @@ if __name__ == '__main__':
     print 'time per cycle', round( 1000*t1/cycles, 2), 'ms'
 
   repeat = 3
-  inch = "1/C6H6/c1-2-3-4-5-6-1/h1-6H"
+  inch = "InChI=1/C6H10/c1-3-5-6-4-2/h3-6H,1-2H3/b5-3-,6-4+" #1/C6H6/c1-2-3-4-5-6-1/h1-6H"
   print "oasa::INCHI DEMO"
   print "converting following inchi into smiles (%d times)" % repeat
   print "  inchi:   %s" % inch
