@@ -1,6 +1,6 @@
 #--------------------------------------------------------------------------
 #     This file is part of OASA - a free chemical python library
-#     Copyright (C) 2003 Beda Kosata <beda@zirael.org>
+#     Copyright (C) 2003-2008 Beda Kosata <beda@zirael.org>
 
 #     This program is free software; you can redistribute it and/or modify
 #     it under the terms of the GNU General Public License as published by
@@ -40,35 +40,43 @@ class molecule( graph.graph):
     # aliases
     self.atoms = self.vertices
     self.bonds = self.edges
-
-
+    self.stereochemistry = []
 
   def __str__( self):
     return "molecule, %d atoms, %d bonds" % (len( self.vertices), len( self.edges))
 
-
-
-
   def create_vertex( self, vertex_class=None):
     if not vertex_class:
       return atom()
     else:
       return vertex_class()
 
-
-
   def create_edge( self):
     return bond()
 
-
-
   def create_graph( self):
     import config
     return config.Config.molecule_class()
 
+  def add_stereochemistry( self, stereo):
+    self.stereochemistry.append( stereo)
+
+  def remove_stereochemistry( self, stereo):
+    if not stereo in self.stereochemistry:
+      raise ValueError, "cannot remove non-existent stereochemistry information"
+    self.stereochemistry.remove( stereo)
 
   # analytics
 
+  # override of graphs method to add stereochemistry support
+  def get_disconnected_subgraphs( self):
+    out = graph.graph.get_disconnected_subgraphs( self)
+    for part in out:
+      for st in self.stereochemistry:
+        if Set( st.references) <= Set( part.vertices):
+          part.add_stereochemistry( st)
+    return out
+
   def get_mol_weight( self):
     w = 0
     for v in self.vertices:
@@ -80,6 +88,11 @@ class molecule( graph.graph):
   weight = property( get_mol_weight, None, None, "molecular weight")
 
 
+  def get_charge( self):
+    return sum( [a.charge for a in self.vertices])
+
+  charge = property( get_charge, None, None, "Net charge of the molecule")
+
 
   def get_formula_dict( self):
     """returns a formula dict as defined in the periodic_table.py::formula_dict"""
@@ -221,14 +234,16 @@ class molecule( graph.graph):
 
   def _get_atoms_possible_aromatic_electrons( self, at, ring):
     out = Set()
-    if at.charge > 0:
+    if at.charge > 0 and at.symbol not in PT.accept_cation:
       out.add( 0)
     elif at.charge < 0:
       out.add( 2)
-    if at.symbol in ('N','S','O','Se','P'):
+    if at.symbol in PT.accept_cation and not at.charge > 0:
       out.add( 2)
     if at.symbol in ('B','Al'):
       out.add( 0)
+    #if at.multiplicity == 2:
+    #  out.add( 1)
     for b, a in at.get_neighbor_edge_pairs():
       if b.order > 1 and (a in ring or b.aromatic):
         out.add( 1)
@@ -356,6 +371,18 @@ class molecule( graph.graph):
       to_go = [b for b in self.bonds if b.order == 4]
 
 
+  def remove_unimportant_hydrogens( self):
+    """removes all H atoms that don't have any special properties
+    or are not part of a hydrogen only molecule."""
+    only_hydrogens = True
+    for v in self.vertices:
+      if v.symbol != "H":
+        only_hydrogens = False
+        break
+    if not only_hydrogens:
+      for v in copy.copy( self.vertices):
+        if v.symbol == 'H' and v.degree <= 1 and not v.isotope:
+          self.remove_vertex( v)
 
 
 
@@ -373,6 +400,12 @@ class molecule( graph.graph):
     i = 0
     while 1:
       out = [a.symbol_number for a in self.vertices if a.properties_['d'] == i]
+      if i > 0:
+        out2 = []
+        for a in self.vertices:
+          if a.properties_['d'] == i-1:
+            out2 += a.get_hydrogen_count() * [1]
+        out += out2
       if out:
         out.sort()
         # out.reverse()
@@ -488,6 +521,7 @@ class molecule( graph.graph):
     add_implicit = False
     for v in other.vertices:
       if (isinstance( v, atom) and v.symbol == 'H') or \
+         (isinstance( v, atom) and v.explicit_hydrogens > 0) or \
          (isinstance( v, query_atom) and ('H' in v.symbols or 'R' in v.symbols)):
         add_implicit = True
         break
@@ -495,23 +529,40 @@ class molecule( graph.graph):
       added_hs = self.add_missing_hydrogens()
       for h in added_hs:
         h.properties_['implicit_hydrogen'] = True
-
+    # here we add explicit hydrogens to other, we call them implicit_hydrogen to make it
+    # work the same way as implicit_hydrogen
+    for v in other.vertices:
+      for i in range( v.explicit_hydrogens):
+        h = other.create_vertex()
+        other.add_vertex( h)
+        h.symbol = 'H'
+        e = other.create_edge()
+        e.order = 1
+        other.add_edge( v, h, e=e)
+        h.properties_['implicit_hydrogen'] = True
+      v.explicit_hydrogens = 0  # make it 0 and after search put it back
+    # for implicit_freesites we add the free_sites here
+    if implicit_freesites:
+      for v in other.vertices:
+        v.properties_['old_free_sites'] = v.free_sites
+        v.free_sites = v.free_valency
 
     # then we create the dicts for storing threads in each of the atoms
     i = 0 
     for a in other.vertices:
-      a.properties_[ 'subsearch'] = {}
+      a.properties_['subsearch'] = {}
+    for e in other.edges | self.edges:
+      e.properties_['subsearch'] = {}
     # here we select the vertex to start from
     vs = [v for v in other.vertices if isinstance( v, atom)]
     sym = common.least_common_item( [v.symbol for v in vs])
     v = [v for v in vs if v.symbol == sym][0]
-
     for a in self.vertices:
-      a.properties_[ 'subsearch'] = {}
+      a.properties_['subsearch'] = {}
       if v.matches( a):
         i += 1
-        a.properties_[ 'subsearch'][i] = v
-        v.properties_[ 'subsearch'][i] = a
+        a.properties_['subsearch'][i] = v
+        v.properties_['subsearch'][i] = a
 
     # now we can proceed with the search
     yielded = Set()
@@ -520,11 +571,7 @@ class molecule( graph.graph):
       # for symetrical fragments we have to get rid of copies (O1=N=O2 and O2=N=O1)
       vsset = ImmutableSet( vs)
       if vsset not in yielded:
-        # if we do not use implicit_freesites we have to check the non-implicit one here
-        if not implicit_freesites:
-          if self._freesites_match( other, thread):
-            yield [v for v in vs if not 'implicit_hydrogen' in v.properties_.keys()]
-        else:
+        if self._freesites_match( other, thread):
           yield [v for v in vs if not 'implicit_hydrogen' in v.properties_.keys()]
       yielded.add( vsset)
 
@@ -534,16 +581,29 @@ class molecule( graph.graph):
 
 
   def clean_after_search( self, other):
+    # for implicit_freesites we restore original free_sites here
+    for v in other.vertices:
+      if "old_free_sites" in v.properties_:
+        v.free_sites = v.properties_['old_free_sites']
+        del v.properties_['old_free_sites']
     # finally we remove the added implicit hydrogens
     hs = [v for v in self.vertices if 'implicit_hydrogen' in v.properties_.keys()]
     for v in hs:
       del v.properties_['implicit_hydrogen']
       self.remove_vertex( v)
-
+    # we also remove implicit_hydrogen (that represent atom.explicit_hydrogens) on other
+    # and also set the explicit_hydrogens count back to the original atoms
+    for v in other.vertices:
+      if 'implicit_hydrogen' not in v.properties_:
+        v.explicit_hydrogens = len( [h for h in v.neighbors if 'implicit_hydrogen' in h.properties_])
+    hs = [v for v in other.vertices if 'implicit_hydrogen' in v.properties_.keys()]
+    for v in hs:
+      del v.properties_['implicit_hydrogen']
+      other.remove_vertex( v)
     for v in self.vertices + other.vertices:
       del v.properties_['subsearch']
-
-
+    for e in self.edges | other.edges:
+      del e.properties_['subsearch']
     
       
   def _mark_matching_threads( self, v, other):
@@ -556,11 +616,11 @@ class molecule( graph.graph):
 
       mirror = v.properties_['subsearch'][thread]
       for e, n in v.get_neighbor_edge_pairs():
-        if thread not in n.properties_['subsearch'].keys():
+        if thread not in n.properties_['subsearch']:
           candidates = Set()
           for me, mn in mirror.get_neighbor_edge_pairs():
-            if thread not in mn.properties_['subsearch'].keys() and n.matches( mn) and e.matches( me):
-              candidates.add( mn)
+            if thread not in mn.properties_['subsearch'] and mn.matches( n) and me.matches( e) and thread not in e.properties_['subsearch']:
+              candidates.add( (mn, me, e))
 
           if candidates:
             if len( candidates) > 1:
@@ -568,12 +628,14 @@ class molecule( graph.graph):
               ths = [thread] + new_threads
             else:
               ths = [thread]
-            for c in candidates:
+            for c, me, e in candidates:
               th = ths.pop()
               n.properties_['subsearch'][th] = c
               c.properties_['subsearch'][th] = n
+              e.properties_['subsearch'][th] = me
+              me.properties_['subsearch'][th] = e
             [x for x in self._mark_matching_threads( n, other)] # just make the generator run
-            if thread not in v.properties_['subsearch'].keys():
+            if thread not in v.properties_['subsearch']:
               # the thread already died
               break
             else:
@@ -582,15 +644,16 @@ class molecule( graph.graph):
             self._delete_thread( other, thread)
             break
         # for proper handling of rings we have to check also the ones that are in this thread already
-        else: 
-          found = False
-          for me, mn in mirror.get_neighbor_edge_pairs():
-            if thread in mn.properties_['subsearch'].keys() and n.properties_['subsearch'][thread] == mn and e.matches( me):
-              found = True
-              break
-          if not found:
+        elif thread not in e.properties_['subsearch']:
+          me = self.get_edge_between( mirror, n.properties_['subsearch'][thread]) 
+          if me and e.matches( me):
+            e.properties_['subsearch'][thread] = me
+            me.properties_['subsearch'][thread] = e
+          else:
             self._delete_thread( other, thread)
             break
+        else:
+          pass
          
       threads = [i for i in v.properties_['subsearch'].keys() if i >= thread]
       if thread in threads:
@@ -600,11 +663,13 @@ class molecule( graph.graph):
 
 
   def _spawn_thread( self, other, thread, number):
-    my_vs = [v for v in self.vertices if thread in v.properties_['subsearch'].keys()]
-    other_vs = [v for v in other.vertices if thread in v.properties_['subsearch'].keys()]
-    max_thread = max( [max( v.properties_['subsearch'].keys()) for v in other.vertices if v.properties_['subsearch'].keys()])
+    my_vs = [v for v in self.vertices if thread in v.properties_['subsearch']]
+    my_es = [e for e in self.edges if thread in e.properties_['subsearch']]
+    other_vs = [v for v in other.vertices if thread in v.properties_['subsearch']]
+    other_es = [e for e in other.edges if thread in e.properties_['subsearch']]
+    max_thread = max( [max( v.properties_['subsearch'].keys()) for v in other.vertices if v.properties_['subsearch']])
     for i in range( max_thread +1, max_thread +number +1, 1):
-      for v in my_vs + other_vs:
+      for v in my_vs + my_es + other_vs + other_es:
         v.properties_['subsearch'][ i] = v.properties_['subsearch'][thread]
     return range( max_thread +1, max_thread +number +1, 1)
 
@@ -621,12 +686,12 @@ class molecule( graph.graph):
   def _freesites_match( self, other, thread):
     for v in other.vertices:
       mirror = v.properties_['subsearch'][thread]
-      unmatched_ns = [n for n in mirror.neighbors if thread not in n.properties_['subsearch'] and not n.symbol == 'H']
+      unmatched_ns = [n for n in mirror.neighbors if thread not in n.properties_['subsearch']] # and not n.symbol == 'H']
       # if there is more unmatched neighbors then free-site it does not match (Hs don't count)
-      if not len( unmatched_ns) <= v.free_sites:
+      if not len( unmatched_ns)+mirror.explicit_hydrogens <= v.free_sites:
         return False
     return True
-        
+
 
   def contains_substructure( self, other, implicit_freesites=True):
     found = False
@@ -639,6 +704,70 @@ class molecule( graph.graph):
 
   # // --- end of the fragment matching routines ---
 
+  def detect_stereochemistry_from_coords( self, omit_rings=True):
+    import stereochemistry,geometry
+    def add_neighbor_double_bonds( bond, path):
+      for _e in bond.get_neighbor_edges():
+        if _e.order == 2 and _e not in path:
+          path.append( _e)
+          add_neighbor_double_bonds( _e, path)
+    # double bonds
+    # detect clusters of double bonds
+    double_paths = []
+    processed = Set()
+    for e in self.edges:
+      if e.order == 2 and e not in processed:
+        if omit_rings and not self.is_edge_a_bridge( e):
+          continue
+        path = [e]
+        add_neighbor_double_bonds( e, path)
+        if len( path) % 2:
+          double_paths.append( path)
+          processed |= Set( path)
+    # detect config on these paths
+    for path in double_paths:
+        vertices = []
+        for bond in path:
+          vertices.extend( bond.vertices)
+        ends = [v for v in vertices if vertices.count(v) == 1]
+        if len( ends) != 2: # two ends is the only thing we are prepared to handle
+          continue
+        end1, end2 = ends
+        # set stereochemistry for all neighbors of both ends
+        for e1,n1 in end1.get_neighbor_edge_pairs():
+          plane1 = geometry.plane_normal_from_3_points( (n1.x,n1.y,n1.z),(end1.x,end1.y,end1.z),(end2.x,end2.y,end2.z))
+          if plane1 == None:
+            continue # some coords were missing
+          if not e1 in path:
+            for e2,n2 in end2.get_neighbor_edge_pairs():
+              if not e2 in path:
+                plane2 = geometry.plane_normal_from_3_points( (end1.x,end1.y,end1.z),(end2.x,end2.y,end2.z),(n2.x,n2.y,n2.z))
+                #cos_angle = geometry.same_or_oposite_side( plane1, plane2)
+                cos_angle = geometry.angle_between_planes( plane1, plane2)
+                if cos_angle < 0:
+                  value = stereochemistry.cis_trans_stereochemistry.OPPOSITE_SIDE
+                else:
+                  value = stereochemistry.cis_trans_stereochemistry.SAME_SIDE
+                if len( path) == 1:
+                  center = path[0]
+                else:
+                  center = None
+                refs = [n1,end1,end2,n2]
+                st = stereochemistry.cis_trans_stereochemistry( center=center, value=value, references=refs)
+                to_remove = None
+                to_add = None
+                for st1 in self.stereochemistry:
+                  if Set( st1.references) == Set( st.references):
+                    if st.value == st1.value:
+                      break
+                    else:
+                      to_remove = st1
+                      break
+                else:
+                  self.add_stereochemistry( st)
+                if to_remove:
+                  self.remove_stereochemistry( to_remove)
+
 
   def mark_morgan( self):
     old_morgs = 0
@@ -680,7 +809,6 @@ class molecule( graph.graph):
       if not maxy or v.y > maxy:
         maxy = v.y
 
-
     scale = bond_length / self.get_mean_bond_length()
     movex = (maxx+minx)/2
     movey = (maxy+miny)/2
@@ -695,6 +823,8 @@ class molecule( graph.graph):
 
   def get_mean_bond_length( self):
     """returns the mean bond length of bonds in the molecule"""
+    if len( self.edges) == 0:
+      return None
     bond_length = 0.0
     for e in self.edges:
       v1, v2 = e.vertices
@@ -702,6 +832,17 @@ class molecule( graph.graph):
     bl = bond_length / len( self.edges)
     return bl
 
+  def get_structure_hash( self):
+    vs = self.number_atoms_uniquely()
+    ret = []
+    for v in vs:
+      ret.append( str(v.properties_['distance_matrix']))
+    res = "*".join( ret)
+    import sha
+    ss = sha.new()
+    ss.update( res)
+    return ss.hexdigest()
+
 
 def the_right_sorting_function( t1, t2):
   for i,l in enumerate( t1):
