@@ -1,6 +1,6 @@
 #--------------------------------------------------------------------------
 #     This file is part of OASA - a free chemical python library
-#     Copyright (C) 2003 Beda Kosata <beda@zirael.org>
+#     Copyright (C) 2003-2008 Beda Kosata <beda@zirael.org>
 
 #     This program is free software; you can redistribute it and/or modify
 #     it under the terms of the GNU General Public License as published by
@@ -21,6 +21,9 @@ from plugin import plugin
 from molecule import molecule, equals
 import periodic_table as PT
 import oasa_exceptions
+import reaction
+import stereochemistry
+
 
 from config import Config
 
@@ -35,7 +38,7 @@ class smiles( plugin):
   read = 1
   write = 1
 
-  smiles_to_oasa_bond_recode = {'-': 1, '=': 2, '#': 3, ':': 4}
+  smiles_to_oasa_bond_recode = {'-': 1, '=': 2, '#': 3, ':': 4, ".": 0, "\\": 1, "/": 1}
   oasa_to_smiles_bond_recode = {1: '', 2: '=', 3: '#', 4:''}
 
   def __init__( self, structure=None):
@@ -44,6 +47,30 @@ class smiles( plugin):
   def recode_oasa_to_smiles_bond( self, b):
     if b.aromatic:
       return ''
+    elif b in self._stereo_bonds_to_others:
+      others = [(e,st) for e,st in self._stereo_bonds_to_others[b] if e in self._stereo_bonds_to_code]
+      if not others:
+        code = "\\"
+      else:
+        # other bonds enforce encoding of this one, we select the first one,
+        # bacause there is nothing we can do if there are clashing constrains anyway
+        other, st = others[0]
+        end1,inside1,inside2,end2 = st.references
+        if Set( other.vertices) == Set( [end1,inside1]):
+          v1 = inside1
+          v2 = end1
+        else:
+          v1 = inside2
+          v2 = end2
+        last_order = self._processed_atoms.index( v2) - self._processed_atoms.index( v1)
+        last_code = self._stereo_bonds_to_code[ other] == "\\" and 1 or -1
+        relation = st.value == st.OPPOSITE_SIDE and -1 or 1
+        if relation*last_code*last_order < 0:
+          code = "/"
+        else:
+          code = "\\"
+      self._stereo_bonds_to_code[ b] = code
+      return code
     else:
       if b.order == 1:
         a1, a2 = b.vertices
@@ -62,64 +89,66 @@ class smiles( plugin):
     mol = Config.create_molecule()
     text = "".join( text.split())
     is_text = re.compile("^[A-Z][a-z]?$")
-    is_small_text = re.compile( '^[a-z]')
-    is_numer = re.compile("[0-9]")
-    chunks = re.split( "([A-Z][a-z]?|[^A-Z]|[a-z])", text)
-    chunks = filter( None, chunks)
-    self._check_the_chunks( chunks)
+    # internally revert \/ bonds before numbers, this makes further processing much easier
+    text = re.sub( r"([\\/])([0-9])", lambda m: (m.group(1)=="/" and "\\" or "/")+m.group(2), text)
+    # // end
+    chunks = re.split( "(\[.*?\]|[A-Z][a-z]?|%[0-9]{1,2}|[^A-Z]|[a-z])", text)
+    chunks = self._check_the_chunks( chunks)
     last_atom = None
     last_bond = None
     numbers = {}
     bracket_openings = []
     for c in chunks:
       # atom
-      if is_text.match( c) or is_small_text.match( c):
-        if is_small_text.match( c):
-          symbol = c.upper()
-        else:
-          symbol = c
+      if is_text.match( c) or c.islower() or c[0] == "[":
         a = mol.create_vertex()
-        a.symbol = symbol
+        if c[0] == "[":
+          # atom spec in square brackets
+          self._parse_atom_spec( c, a)
+        else:
+          # just atom symbol
+          if c.islower():
+            symbol = c.upper()
+            a.properties_['aromatic'] = 1
+          else:
+            symbol = c
+          a.symbol = symbol
+
         mol.add_vertex( a)
-        if last_bond and not (not is_small_text.match( c) and last_bond.aromatic):
+        if last_bond: # and not (not 'aromatic' in a.properties_ and last_bond.aromatic):
           mol.add_edge( last_atom, a, e=last_bond)
           last_bond = None
         elif last_atom:
           b = mol.add_edge( last_atom, a)
-          if is_small_text.match( c):
+          if 'aromatic' in a.properties_:
             # aromatic bond
             b.order = 4
             b.type = 'n'
-            a.properties_['aromatic'] = 1  #we need this bellow
         last_atom = a
-        if is_small_text.match( c):
-          # aromatic bond
-          last_bond = mol.create_edge()
-          last_bond.order = 4
-          last_bond.type = 'n'
-          last_atom.properties_['aromatic'] = 1  #we need this bellow
-        else:
-          last_bond = None
+        last_bond = None
       # bond
-      elif c in '-=#:.':
+      elif c in r'-=#:.\/':
         order = self.smiles_to_oasa_bond_recode[ c]
         last_bond = mol.create_edge()
         last_bond.order = order
         last_bond.type = 'n'
+        if c in r'\/':
+          last_bond.properties_['stereo'] = c
       # ring closure
-      elif is_numer.match( c):
-        b = last_bond or mol.create_edge()
+      elif c.isdigit():
         if c in numbers:
-          mol.add_edge( last_atom, numbers[c], e=b)
-          if 'aromatic' in last_atom.properties_:
-            last_bond = mol.create_edge()
-            last_bond.order = 4
-            last_bond.type = 'n'
+          if last_bond:
+            b = last_bond
           else:
-            last_bond = None
+            b = mol.create_edge()
+            if "aromatic" in numbers[c].properties_:
+              b.order = 4
+          mol.add_edge( last_atom, numbers[c], e=b)
+          last_bond = None
           del numbers[ c]
         else:
           numbers[c] = last_atom
+          last_bond = None
       elif c == '(':
         bracket_openings.append( last_atom)
       elif c == ')':
@@ -127,36 +156,163 @@ class smiles( plugin):
 
     ## FINISH
     for a in mol.vertices:
-      a.raise_valency_to_senseful_value()
+      if not "explicit_valency" in a.properties_:
+        a.raise_valency_to_senseful_value()
+      else:
+        # detect radicals (but not biradicals - problem of triplet vs. singlet)
+        del a.properties_['explicit_valency']
+        if a.valency - a.occupied_valency == 1:
+          a.multiplicity += 1
+        else:
+          a.valency = a.occupied_valency
       try:
         del a.properties_['aromatic']
       except:
         pass
 
+    # stereochemistry
+    self._process_stereochemistry( mol)
+
+    if len(mol.vertices) == 0:
+      mol = None
     self.structure = mol 
 
+
+  def _parse_atom_spec( self, c, a):
+    """c is the text spec,
+    a is an empty prepared vertex (atom) instance"""
+    bracketed_atom = re.compile("^\[(\d*)([A-z][a-z]?)(.*?)\]")
+    m = bracketed_atom.match( c)
+    if m:
+      isotope, symbol, rest = m.groups()
+    else:
+      raise ValueError( "unparsable square bracket content '%s'" % c)
+    if symbol.islower():
+      symbol = symbol.upper()
+      a.properties_['aromatic'] = 1
+    a.symbol = symbol
+    if isotope:
+      a.isotope = int( isotope)
+    # hydrogens
+    _hydrogens = re.search( "H(\d*)", rest)
+    h_count = 0
+    if _hydrogens:
+      if _hydrogens.group(1):
+        h_count = int( _hydrogens.group(1))
+      else:
+        h_count = 1
+    a.explicit_hydrogens = h_count
+    # charge
+    charge = 0
+    # one possible spec of charge
+    _charge = re.search( "[-+]{2,10}", rest)
+    if _charge:
+      charge = len( _charge.group(0))
+      if _charge.group(0)[0] == "-":
+        charge *= -1
+    # second one, only if the first one failed
+    else:
+      _charge = re.search( "([-+])(\d?)", rest)
+      if _charge:
+        if _charge.group(2):
+          charge = int( _charge.group(2))
+        else:
+          charge = 1
+        if _charge.group(1) == "-":
+          charge *= -1
+    a.charge = charge
+    # stereo
+    _stereo = re.search( "@+", rest)
+    if _stereo:
+      stereo = _stereo.group(0)
+      a.properties_['stereo'] = stereo
+    # using [] means valency is explicit
+    a.properties_['explicit_valency'] = True
+
   def _check_the_chunks( self, chunks):
     is_text = re.compile("^[A-Z][a-z]?$")
-    i = 0
-    while i < len( chunks):
-      c = chunks[i]
+    is_long_num = re.compile( "^%[0-9]{1,2}$")
+    ret = []
+    for c in chunks:
+      if not c:
+        continue
       if is_text.match( c):
-        if not c in PT.periodic_table or c == "Sc": # Sc is S-c not scandium
+        if (not c in PT.periodic_table and len(c)==2) or c == "Sc": # Sc is S-c not scandium
           a,b = c
-          del chunks[i]
-          chunks.insert( i, b)
-          chunks.insert( i, a)
-          i += 1
-      i += 1
+          ret.append( a)
+          ret.append( b)
+        else:
+          ret.append( c)
+      elif is_long_num.match( c):
+        ret.append( str( int( c[1:])))
+      else:
+        ret.append( c)
+    return ret
 
 
 
+  def _process_stereochemistry( self, mol):
+    ## process stereochemistry
+    ## double bonds
+    def get_stereobond_direction( end_atom, inside_atom, bond, init):
+      position = mol.vertices.index( end_atom) - mol.vertices.index( inside_atom)
+      char = bond.properties_['stereo'] == "\\" and 1 or -1
+      direction = (position * char * init) < 0 and "up" or "down"
+      return direction
+    def get_end_and_inside_vertex_from_edge_path( edge, path):
+      a1,a2 = edge.vertices
+      if len( [e for e in a1.neighbor_edges if e in path]) == 1:
+        return a1, a2
+      return a2, a1
+    
+    stereo_edges = [e for e in mol.edges if "stereo" in e.properties_]
+    paths = []
+    for i,e1 in enumerate( stereo_edges):
+      for e2 in stereo_edges[i+1:]:
+        path = mol.get_path_between_edges( e1, e2)
+        path2 = path[1:-1]
+        if len( path2)%2 and not [_e for _e in path2 if _e.order != 2]:
+          # only odd number of double bonds, double bonds only
+          for _e in path[1:-1]:
+            if not mol.is_edge_a_bridge_fast_and_dangerous( _e):
+              break
+          else:
+            # only stereo related to non-cyclic bonds
+            paths.append( path)
+    
+    for path in paths:
+      bond1 = path[0]
+      end_atom1,inside_atom1 = get_end_and_inside_vertex_from_edge_path( bond1, path)
+      bond2 = path[-1]
+      end_atom2,inside_atom2 = get_end_and_inside_vertex_from_edge_path( bond2, path)
+      d1 = get_stereobond_direction( end_atom1, inside_atom1, bond1, -1)
+      d2 = get_stereobond_direction( end_atom2, inside_atom2, bond2, -1)
+      if d1 == d2:
+        value = stereochemistry.cis_trans_stereochemistry.SAME_SIDE
+      else:
+        value = stereochemistry.cis_trans_stereochemistry.OPPOSITE_SIDE
+      if len( path) == 3:
+        center = path[1]
+      else:
+        center = None
+      refs = [end_atom1,inside_atom1,inside_atom2,end_atom2]
+      st = stereochemistry.cis_trans_stereochemistry( center=center, value=value, references=refs)
+      mol.add_stereochemistry( st)
+
+    for e in mol.edges:
+      if 'stereo' in e.properties_:
+        del e.properties_['stereo']
+
+
   def get_smiles( self, mol):
     if not mol.is_connected():
       raise oasa_exceptions.oasa_not_implemented_error( "SMILES", "Cannot encode disconnected compounds, such as salts etc.")
     #mol = molec.copy()
     self.ring_joins = []
+    self._processed_atoms = []
     self.branches = {}
+    self._stereo_bonds_to_code = {} # for bond it will contain character it uses
+    self._stereo_bonds_to_others = {} # for bond it will contain the other bonds
     # at first we mark all the atoms with aromatic bonds
     # it is much simple to do it now when all the edges are present
     # we can make use of the properties attribute of the vertex
@@ -164,13 +320,20 @@ class smiles( plugin):
       if e.aromatic:
         for v in e.vertices:
           v.properties_[ 'aromatic'] = 1
+    # stereochemistry information preparation
+    for st in mol.stereochemistry:
+      end1, inside1, inside2, end2 = st.references
+      e1 = end1.get_edge_leading_to( inside1)
+      e2 = end2.get_edge_leading_to( inside2)
+      self._stereo_bonds_to_others[ e1] = self._stereo_bonds_to_others.get( e1, []) + [(e2, st)]
+      self._stereo_bonds_to_others[ e2] = self._stereo_bonds_to_others.get( e2, []) + [(e1, st)]
+
     ret = ''.join( [i for i in self._get_smiles( mol)])
     mol.reconnect_temporarily_disconnected_edges()
     # this is needed because the way temporarily_disconnected edges are handled is not compatible with the way smiles
-    # generation failed - it splits the molecule while reusing the same atoms and bonds and thus disconnected bonds accounting fails
+    # generation works - it splits the molecule while reusing the same atoms and bonds and thus disconnected bonds accounting fails
     for e in mol.edges:
       e.disconnected = False
-
     return ret
 
 
@@ -179,10 +342,7 @@ class smiles( plugin):
     # single atoms
     if len( mol.vertices) == 1:
       v = mol.vertices[0]
-      if 'aromatic' in v.properties_.keys():
-        yield v.symbol.lower()
-      else:
-        yield v.symbol
+      yield self._create_atom_smiles( v)
       for e in self.ring_joins:
         if v in e.get_vertices():
           yield self.recode_oasa_to_smiles_bond( e)
@@ -212,15 +372,15 @@ class smiles( plugin):
       start, end = end, start
     v = start
     last = None
-    while v != end:
-      if 'aromatic' in v.properties_.keys():
-        yield v.symbol.lower()
-      else:
-        yield v.symbol
+    was_end = False
+    while True:
+      yield self._create_atom_smiles( v)
       # the atom
       for e in self.ring_joins:
         if v in e.get_vertices():
-          yield self.recode_oasa_to_smiles_bond( e)
+          _b = self.recode_oasa_to_smiles_bond( e)
+          if _b not in "/\\":
+            yield _b
           yield str( self.ring_joins.index( e) +1)
       # branches
       if v in self.branches:
@@ -239,29 +399,40 @@ class smiles( plugin):
           last = v
           v = neighbor
           break
-    # the last atom - should make it somehow not to need this piece of code
+      if was_end:
+        break
+      if v == end:
+        was_end = True
+
+
+  def _create_atom_smiles( self, v):
+    self._processed_atoms.append( v)
     if 'aromatic' in v.properties_.keys():
-      yield v.symbol.lower()
+      symbol = v.symbol.lower()
     else:
-      yield v.symbol
-    for e in self.ring_joins:
-      if v in e.get_vertices():
-        yield self.recode_oasa_to_smiles_bond( e)
-        yield str( self.ring_joins.index( e) +1)
-      if v in self.branches:
-        for edg, branch in self.branches[ v]:
-          yield '('
-          yield self.recode_oasa_to_smiles_bond( edg)
-          v1, v2 = edg.vertices
-          vv = (v1 != v) and v1 or v2
-          for i in self._get_smiles( branch, start_from=vv):
-            yield i
-          yield ')'
-
-
-
-
+      symbol = v.symbol
 
+    if v.isotope or v.charge != 0 or v.valency != PT.periodic_table[ v.symbol]['valency'][0] or 'stereo' in v.properties_ or v.multiplicity != 1:
+      # we must use square bracket
+      isotope = v.isotope and str( v.isotope) or ""
+      # charge
+      if v.charge:
+        sym = v.charge < 0 and "-" or "+"
+        charge = sym + (abs( v.charge) > 1 and str( abs( v.charge)) or "")
+      else:
+        charge = ""
+      # explicit hydrogens
+      if v.explicit_hydrogens or v.valency != PT.periodic_table[ v.symbol]['valency'][0]:
+        num_h = v.valency - v.occupied_valency
+        h_spec = (num_h and "H" or "") + (num_h > 1 and str( num_h) or "")
+      else:
+        h_spec = ""
+      # stereo
+      stereo = v.properties_.get( "stereo", "")
+      return "[%s%s%s%s%s]" % (isotope, symbol, stereo, h_spec, charge)
+    else:
+      # no need to use square brackets
+      return symbol
 
 
   def disconnect_something( self, mol, start_from=None):
@@ -400,8 +571,114 @@ def is_line( mol):
 def is_pure_ring( mol):
   return filter( lambda x: x.get_degree() != 2, mol.vertices) == []
 
+
 ##################################################
-## MODULE INTERFACE
+## MODULE INTERFACE - newstyle
+
+from converter_base import converter_base
+
+class smiles_converter( converter_base):
+
+  # standard converter attrs
+  reads_text = True
+  writes_text = True
+  reads_files = True
+  writes_files = True
+
+  default_configuration = {"R_GENERATE_COORDS": True,
+                           "R_BOND_LENGTH": 1,
+                           "R_LOCALIZE_AROMATIC_BONDS": True,
+                           
+                           "W_AROMATIC_BOND_AUTODETECT": True,
+                           "W_INDIVIDUAL_MOLECULE_SEPARATOR": ".",
+                           "W_DETECT_STEREO_FROM_COORDS": True,
+                           }
+
+  def __init__( self):
+    converter_base.__init__( self)
+
+  def mols_to_text( self, structures):
+    converter_base.mols_to_text( self, structures)
+    sm = smiles()
+    ret = []
+    for mol in structures:
+      if self.configuration["W_AROMATIC_BOND_AUTODETECT"]:
+        mol.mark_aromatic_bonds()
+      if self.configuration["W_DETECT_STEREO_FROM_COORDS"]:
+        mol.detect_stereochemistry_from_coords()
+      ret.append( sm.get_smiles( mol))
+    self.last_status = self.STATUS_OK
+    return self.configuration["W_INDIVIDUAL_MOLECULE_SEPARATOR"].join( ret)
+
+  def read_text( self, text):
+    # reaction support
+    if ">" in text:
+      if text.count(">") != 2:
+        raise oasa_exceptions.oasa_smiles_error( "Only two '>' characters are supported in SMILES reactions. %d present" % text.count(">"))
+      parts = text.split( ">")
+      res = []
+      for part in parts:
+        res.append( self._read_string( part))
+        if not self.last_status == self.STATUS_OK:
+          res = []
+          break
+      if not res:
+        self.result = []
+        return []
+      else:
+        react = reaction.reaction()
+        for p in res[0]:
+          react.reactants.append( reaction.reaction_component( p))
+        for p in res[2]:
+          react.products.append( reaction.reaction_component( p))
+        for p in res[1]:
+          react.reagents.append( reaction.reaction_component( p))
+        self.result = [react]
+        return self.result
+    else:
+      return self._read_string( text)
+
+  def _read_string( self, text):
+    converter_base.read_text( self, text)
+    sm = smiles()
+    sm.read_smiles( text)
+    mol = sm.structure
+    if mol == None:
+      return []
+    zero_bonds = [e for e in mol.edges if e.order == 0]
+    for b in zero_bonds:
+      mol.disconnect_edge( b)
+    mols = mol.get_disconnected_subgraphs()
+    for mol in mols:
+      if self.configuration["R_LOCALIZE_AROMATIC_BONDS"]:
+        mol.localize_aromatic_bonds()
+        for b in mol.bonds:
+          b.aromatic = 0
+      if self.configuration["R_GENERATE_COORDS"]:
+        coords_generator.calculate_coords( mol, bond_length=self.configuration['R_BOND_LENGTH'])
+    self.result = mols
+    self.last_status = self.STATUS_OK
+    return mols
+
+  def mols_to_file( self, structures, f):
+    converter_base.mols_to_file( self, structures, f)
+    f.write( self.mols_to_text( structures))
+
+  def read_file( self, f):
+    converter_base.read_file( self, f)
+    mols = []
+    for line in f:
+      mol = self.text_to_mols( line)
+      mols.extend( mol)
+    return mols
+    
+converter = smiles_converter
+
+# END OF MODULE INTERFACE
+##################################################
+
+##################################################
+## MODULE INTERFACE - oldstyle
 
 import coords_generator
 
@@ -447,16 +724,15 @@ if __name__ == '__main__':
 
   def main( text, cycles):
     t = time.time()
-    #mol = molecule()
-    #mol._read_file()
+    conv = converter()
     for j in range( cycles):
-      mol = text_to_mol( text)
-      mol.remove_all_hydrogens()
-      text = mol_to_text( mol)
-      #print mol.get_smallest_independent_cycles_e()
-      print "  generated: %s" % text
-      mol.mark_morgan()
-      #print mol.get_diameter()
+      mols = conv.read_text( text)
+      for mol in mols:
+        mol.remove_unimportant_hydrogens()
+        print "  summary formula:   ", mol.get_formula_dict()        
+      text = conv.mols_to_text( mols)
+      print "  generated SMILES:   %s" % text
+      print "  --"
     t = time.time()-t
     print 'time per cycle', round( 1000*t/cycles, 2), 'ms'
 
@@ -464,12 +740,14 @@ if __name__ == '__main__':
 
   if not len( sys.argv) > 1:
     text = "COc5ccc4c2sc(cc2nc4c5)-c(cc1nc3c6)sc1c3ccc6OC"  #"ccc4ccc2cc1cc3ccccc3cc1cc2c4"
+    text = "C1CC=CC1"
   else:
     text = sys.argv[1]
 
   print "oasa::SMILES DEMO"
   print "converting following smiles to smiles (%d times)" % repeat
-  print "  starting with: %s" % text
+  print "  starting with:      %s" % text
+  print "  --------------------"
   main( text, repeat)
 
 # DEMO END
@@ -488,10 +766,9 @@ if __name__ == '__main__':
 # at first get rid of the edges common to two or more rings, not critical
 # handling of the start_from in disconnect_something as in disconnect_something_simple
 # could in disconnect_something happen that a start_from will be returned in a branch?
-
-# in the reading code the more possibilities will be there the more cleanup (last_bond=None,
-# last_shit=None ...) will be need. Any way around this?
 # if I do "print m.get_smiles( m.structure)" the ring counting does not work after that
 ## the transformation can be destructive!!!  - check
 
 ## THIS IS A PROBLEM : C=1ccC=2C=1C=CC=CC=2  (should be azulene)
+
+# E/Z stereo is ignored in rings
